!#******************************************************************************
!# E.S.O. - VLT project
!#
!#
!# who       when        what
!# --------  ----------  ----------------------------------------------
!# ebarrios  2005-02-04  Addapted from ces Alingment procedure
!#

!#************************************************************************
!#   NAME          EXTsornt
!# 
!#   SYNOPSIS      EXTsornt <image name>.fits
!# 
!#   DESCRIPTION 
!#               Either slit or CCD alignment procedure
!#               Use a calibration light to iluminate any slit.     
!#               Alignment is determined by crosscorrelating segments of 
!#               the spatial profile.
!#
!#.VERSION
!# 100407	last modif
!#------------------------------------------------------------------------

!
! Variable definition:
! --------------------
!
clear/context -all

define/para P1 ? I
indisk/fits {P1} align
! 
read/des {P1}
read/des align
! 
define/local IMAGE/C/1/60 align
define/local I/I/1/1 0
define/local J/I/1/1 0
define/local K/I/1/1 0
define/local REPEAT/I/1/1 0
define/local SCL/I/1/2 -5,-10
define/local WIN/I/1/2 0,0
define/local BIN/R/1/2 0.0,0.0
! 
define/local START/R/1/2 0.0,0.0
define/local NPIX/I/1/2 0,0
define/local XSTART/R/1/1 0.0
define/local XEND/R/1/1 0.0
define/local YSTART/R/1/1 0.0
define/local YEND/R/1/1 0.0
! 
define/local JUMP/I/1/1 0
define/local OK/I/1/1 0
define/local CUT/R/1/2 0.0,0.0
define/local MIS/R/1/3 0.0,0.0,0.0
define/local ERR/R/1/3 0.0,0.0,0.0
define/local ALI/R/1/3 0.0,0.0,0.0
define/local ALIERR/R/1/3 0.0,0.0,0.0
define/local ANG/R/1/1 0.0
define/local ANGERR/R/1/1 0.0
define/local ANGLE/R/1/1 0.0
define/local QUNO/R/1/1 0.0
define/local QDOS/R/1/1 0.0
define/local MEAN/R/1/1 0.0

!
! If display and graphic window don't exist they are created:
! -----------------------------------------------------------
!
create/disp
create/graph
set/graph
set/graph pmode=1 xaxis=auto yaxis=auto

!
! Get Image size and binning
!
COPY/DK {IMAGE} NPIX NPIX
COPY/DK {IMAGE} START START
XSTART = START(1)
XEND   = XSTART + NPIX(1) - 1
YSTART = START(2)
YEND   = YSTART + NPIX(2) - 1
!
! Table for one dimension center/gauss <scanline>,CENTER command
!
create/table CENTER 1 20 null
create/column CENTER :XSTART "px"
create/column CENTER :XEND   "px"
create/column CENTER :XCEN
create/column CENTER :YCEN
create/column CENTER :XERR
create/column CENTER :YERR
create/column CENTER :XSIG 
create/column CENTER :YSIG
create/column CENTER :XFWHM
create/column CENTER :YFWHM 
create/column CENTER :ICENT 
create/column CENTER :STATUS 
! 
write/table CENTER :XSTART @1 {YSTART} 
write/table CENTER :XEND   @1 {YEND}
!
! Input file name:
! ----------------
!
write/out "======================================"
write/out " Image {IMAGE}
write/out "--------------------------------------"
!
!
! Check if image is binned or windowed:
! -------------------------------------
!
copy/dk {IMAGE} STEP BIN
! 
if BIN(1) .gt. 1.0 .or. BIN(2) .gt. 1.0 then
   write/out "#################################################"
   write/out "### Error: binned image.                      ###"
   write/out "### Bin-x = {BIN(1)}; Bin-y = {BIN(2)}.       ###" 
   write/out "### Procedure aborted.                        ###"
   write/out "### Repeat with unbinned file.                ###"
   write/out "#################################################"
   errsum = errsum + 1
   return
endif
! 
statistics/image {IMAGE} option=M
write/keyw rcorr/r/1/2  1.330000e+02,5.705000e+03
@@ kcompare sornt010:rcorr outputr 1,2 0.1
write/keyw icorr/i/1/5  4243600,776,184,364,1019  
@@ kcompare sornt012:icorr outputi 1,5
! 
CUT(1) = outputr(1)
CUT(2) = outputr(2)
if CUT(2) .lt. 1000 .or. CUT(2) .gt. 60000 goto SIGNAL
!
! Display image:
! --------------
!
load/image {IMAGE} scale={SCL(1)},{SCL(2)} cuts={CUT(1)},{CUT(2)}
!
! Create table containing alignment analysis:
! -------------------------------------------
!
crea/table align 4 {K} null
create/column align :x "px"
create/column align :shift "px"
create/column align :error "px"
create/column align :y "px"
! 
write/out "============================="
write/out "=== Extracting  regions ==="
write/out "============================="
! 
K = 0
do I = 100 1600 50
   K = K+1
   J = I + 49
   ! 
   !echo/full	
   write/out "prof{K} = {IMAGE} {I},{J}"
   average/column prof{K} = {IMAGE} @{I},@{J}
   center/moment prof{K},CENTER >Null
   write/table align :y @{K} {outputr(5)} 
   xcorr/ima prof0001 prof{K} corr{K} 80
   !echo/off       	
   ! 
   write/table align :x @{K} {J}
   center/gauss corr{K} >Null
   MIS(1) = outputr(5)
   MIS(2) = outputr(5)/2048.*1000.
   MIS(3) = outputr(5)/2048.*2048.
   ERR(1) = outputr(7)
   ERR(2) = outputr(7)/2048.*1000.
   ERR(3) = outputr(7)/2048.*2048.
   outputr(19) = MIS(3)
   outputr(20) = ERR(3)
   copy/kt outputr/R/19/2 align :shift :error @{K}
enddo
! 
read/table align
! 
! verify table align
! 
statistics/table align :shift >Null
write/keyw rcorr/r/1/4   0.00119019,26.0412,13.036,7.89621   
@@ kcompare sornt020:rcorr outputr 1,4 0.001
statistics/table align :error >Null
write/keyw rcorr/r/1/4   0.0100268,0.0126624,0.0117054,0.000548235 
@@ kcompare sornt022:rcorr outputr 1,4 0.00001
!
! Correct sign:
! -------------
compu/table align :shift = -1.0*:shift
!
! Plot the alignment values together with their errors:
! -----------------------------------------------------
!

statistics/table align :shift
compu/table align :err2 = :error*:error
statistics/table align :shift
if {outputr(1)} .gt. -1.0 .and. {outputr(2)} .lt. 1.0 set/grap yaxis=-1.2,1.2
set/grap xaxis=auto
plot/table align :x :shift
over/err align :x :shift :error ? n
set/grap stype=0 ltype=2
over/table align :x :shift
select/table align :error.gt.0
regress/poly align :shift :x 1
save/regress align fit
select/table align all
compu/regress align :fit = fit(#1)
set/grap stype=0 colour=4 ltype=1
over/table align :x :fit
! 
statistics/tab align :fit >Null
write/keyw rcorr/r/1/4   -26.0629,-0.00902668,-13.036,7.89618  
@@ kcompare sornt030:rcorr outputr 1,4 0.0005
if outputi(2) .ne. 31 then
   errsum = errsum + 1
   return
endif
! 
ALI(1) = (m$value(align,:fit,@{K})-m$value(align,:fit,@1))
QUNO = m$value(align,:err2,@1)+m$value(align,:err2,@{K})
ALIERR(1) = m$sqrt(QUNO)
ALI(2) = ALI(1)*9./10./1843.*1000.
ALIERR(2) = ALIERR(1)*9./10./1843.*1000.
ALI(3) = ALI(1)*9./10./1843.*2048.
ALIERR(3) = ALIERR(1)*9./10./1843.*2048.
! 
ANG = m$atan(ALI(1)/2048.)
QUNO = m$atan((ALI(1)+ALIERR(1))/2048.)-ANG
QDOS = ANG-m$atan((ALI(1)-ALIERR(1))/2048.)
ANGERR = (QUNO+QDOS)/2.
! 
write/keyw rcorr/r/1/10 -
0.0168834,0.00824474,0.0168852,-26.0539,-12.723,-26.0567,-0.728857,0.000472244,0.000472278,0.000472261
outputr(1) = alierr(1)
outputr(2) = alierr(2)
outputr(3) = alierr(3)
outputr(4) = ali(1)
outputr(5) = ali(2)
outputr(6) = ali(3)
outputr(7) = ang
outputr(8) = quno
outputr(9) = qdos
outputr(10) = angerr
@@ kcompare sornt040:rcorr outputr 1,3 0.0001 4,6 0.001 7,10 0.000001
!
create/table horizontal 4 2 null
create/column horizontal :x1 "px"
create/column horizontal :x2 "px"
create/column horizontal :shift1 "px"
create/column horizontal :shift2 "px"
write/table horizontal :x1 @1 0.0
write/table horizontal :x1 @2 1500.0
write/table horizontal :x2 @1 2600.0
write/table horizontal :x2 @2 4100.0
compu/table horizontal :shift1 = +1.0
compu/table horizontal :shift2 = -1.0
set/grap ltype=3 colour=2
over/table horizontal :x1 :shift1
over/table horizontal :x1 :shift2
over/table horizontal :x2 :shift1
over/table horizontal :x2 :shift2
label/grap "Lower Limit" 2050.,-1.0
label/grap "Upper Limit" 2050.,1.0
!
statistics/table align :y
write/keyw rcorr/r/1/4    997.551,1023.6,1010.56,7.8985 
@@ kcompare sornt050:rcorr outputr 1,4 0.01
@@ EXTrotalign {ANG} {outputr(3)} {ALI(3)}
write/keyw rcorr/r/1/5   -0.73,1010.56,-26.06,730,-730  
@@ kcompare sornt052:rcorr outputr 1,3 0.005 4,5 0.1
outputr(1) = alierr(1)
outputr(2) = alierr(2)
outputr(3) = alierr(3)
outputr(4) = ali(1)
outputr(5) = ali(2)
outputr(6) = ali(3)
outputr(7) = ang
outputr(8) = angerr
write/keyw rcorr/r/1/8 -
0.0168834,0.00824474,0.0168852,-26.0539,-12.723,-26.0567,-0.728857,0.000472261
@@ kcompare sornt055:rcorr outputr 1,3 0.0001 4,6 0.001 7,8 0.0001
! 
write/out " Alignment:"
write/out " {ANG} +/- {ANGERR} degrees, i.e."
write/out " {ALI(2)} +/- {ALIERR(2)} px in 1000 px"
write/out " {ALI(3)} +/- {ALIERR(3)} px in 2048 px (whole range)"
return
! 
SIGNAL:
write/out "Check Slit image. Signal should be in the range off 1000 to 60000 ADUs"
outputr(1) = -9999.9
outputr(2) = 0.
outputr(3) = 1000.
outputr(4) = 999.9
outputr(5) = 999.9
errsum = errsum + 1

